/****************************************************************************
 Include Section
*****************************************************************************/

#include "global.h"

/*****************************************************************************
 Functions
******************************************************************************/

.SECTION L1_code;

/*****************************************************************************
 Prototypes
******************************************************************************/

.GLOBAL __ctimer;
//.GLOBAL interrupts_setup;
//.GLOBAL ivtmr_handler;

/****************************************************************************
 Function:              __ctimer
 Description:           generic function for setting up the core timer
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R0
 Global Registers Used: P5 -> initially loaded with SYS_MMR_BASE address
 Global Variables Used: none
 C-Callable :           no
*****************************************************************************/

__ctimer:
    
    link 0;
    [--SP] = ASTAT;
    [--SP] = R0;
    [--SP] = P5;
    IMM32(P5,CORE_MMR_BASE);

    call interrupts_setup;

    R0 = 0;
    [P5 + lo(TSCALE)] = R0;

    call __get_cclk_hz;
    [P5 + lo(TPERIOD)] = R0;
    [P5 + lo(TCOUNT)] = R0;

    R0 =    TAUTORLD    |
            TMPWR       |
            TMREN       ;
    [P5 + lo(TCNTL)] = R0;

    P5 = [SP++];
    R0 = [SP++];
    ASTAT = [SP++];
    unlink;

    rts;
__ctimer.end:


/****************************************************************************
 Function:              interrupts_setup
 Description:           enable IVG6 (IVTMR: Core Timer)

                        GP Interrupt    SIC_IAR value
                        IVG7            0
                        IVG8            1
                        IVG9            2
                        IVG10           3
                        IVG11           4
                        IVG12           5
                        IVG13           6
                        IVG14           7
                        IVG15           8
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R7
 Global Registers Used: P5 -> assumed to contain SYS_MMR_BASE address
 Global Variables Used: none
 C-Callable :           no
*****************************************************************************/

interrupts_setup:

    link 0;
    [--SP] = ASTAT;
    [--SP] = R7;

    /* Set up the interrupt vector for the IVG6 */
    IMM32(R7,ivtmr_handler);
    [P5 + lo(EVT6)] = R7;

    /* Unmask IVG6 interrupt in the IMASK register */
    R7 = [P5 + lo(IMASK)];
    bitset(R7, bitpos(EVT_IVTMR));
    [P5 + lo(IMASK)] = R7;

    R7 = [SP++];
    ASTAT = [SP++];
    unlink;

    rts;
interrupts_setup.end:


/****************************************************************************
 Function:              ivtmr_handler
 Description:           generic IVTMR handler
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        none
 Global Registers Used: none
 Global Variables Used: none
 C-Callable :           no
*****************************************************************************/

ivtmr_handler:
    link 0;
    call __led5_toogle;
    unlink;
    rti;
ivtmr_handler.end:


/*****************************************************************************
 EOF
******************************************************************************/
