/*****************************************************************************
**                                                                          **
**      Name:   bfstartup                                                   **
**                                                                          **
******************************************************************************

(C) Copyright 2007 - Analog Devices, Inc.  All rights reserved.

File Name:      bfstartup.asm

Date Modified:  04/11/08    v1.0 ap-muc

History:        06/14/06    v0

Tested
Software:       VisualDSP++ 4.0 (Updated June 2006)
                VisualDSP++ 4.5 (Updated Nov. 2006)
                VisualDSP++ 4.5 (Updated Feb. 2007)
                VisualDSP++ 5.0
                VisualDSP++ 5.0.1.0 (Update 1)
                VisualDSP++ 5.0.2.0 (Update 2)

Tested
Hardware:       ADSP-BF527 EZ-KIT Lite Rev. 1.0 Silicon Rev. 0.0
                ADSP-BF533 EZ-KIT Lite Rev. 1.7 Silicon Rev. 0.4
                ADSP-BF533 EZ-KIT Lite Rev. 2.1 Silicon Rev. 0.5
                ADSP-BF537 EZ-KIT Lite Rev. 1.3 Silicon Rev. 0.2
                ADSP-BF538 EZ-KIT Lite Rev. 1.2 Silicon Rev. 0.3
                ADSP-BF548 EZ-KIT Lite Rev. 1.2 Silicon Rev. 0.0
                ADSP-BF548 EZ-KIT Lite Rev. 1.3 Silicon Rev. 0.0
                ADSP-BF548 EZ-KIT Lite Rev. 2.1 Silicon Rev. 0.1
                ADSP-BF561 EZ-KIT Lite Rev. 1.3 Silicon Rev. 0.3
                ADSP-BF561 EZ-KIT Lite Rev. 2.1 Silicon Rev. 0.5

Supported
Plattforms:     ADSP-BF527 EZ-KIT Lite
                ADSP-BF533 EZ-KIT Lite Rev 1.6 and below (modifications required, only 32MB SDRAM installed)
                ADSP-BF533 EZ-KIT Lite Rev 1.7 and above
                ADSP-BF537 EZ-KIT Lite
                ADSP-BF538 EZ-KIT Lite
                ADSP-BF548 EZ-KIT Lite
                ADSP-BF561 EZ-KIT Lite (Silicon Rev. 0.2 and beyond)

Connections:    ADDS-HPUSB-ICE

Settings:       Default

Purpose:        Generic Startup file for
                ADSP-BF527 EZ-KIT LITE
                ADSP-BF533 EZ-KIT LITE
                ADSP-BF537 EZ-KIT LITE
                ADSP-BF538 EZ-KIT LITE
                ADSP-BF548 EZ-KIT LITE
                ADSP-BF561 EZ-KIT LITE
                tests.

*****************************************************************************/


/****************************************************************************
 Include Section
*****************************************************************************/

#include "configurations.h"
#include <blackfin.h>

#include "main.h"
#include "macros.h"
#include "system.h"
#include "libdsp.h"

#if ( defined (__ADSPBF52x__) && ( __SILICON_REVISION__ >= 0x002 ) )  || ( defined (__ADSPBF54x__) && ( __SILICON_REVISION__ >= 0x001 ) )
    #include <bfrom.h>
    .import "bfrom.h";
#endif

#if defined INSTR_CACHE_EN
    #include "ICPLB.h"
#endif
#if defined DATA_CACHE_EN
    #include "DCPLB.h"
#endif


/*****************************************************************************
 Symbolic constants / definitions
******************************************************************************/

#define CORE_MMR_BASE           0xFFE00000
#define SYS_MMR_BASE            0xFFC00000

#if defined EN_SDRAM
    #if defined __ADSPBF54x__
        #define EN_DDRSDRAM
    #else
        #define EN_SDRSDRAM
    #endif
#endif


/****************************************************************************
 Include Section: System Initialization
*****************************************************************************/

#if defined __ADSPBF527__
    #include "ezkitBF527_initcode.h"
#endif

#if defined __ADSPBF533__
    /*************************************
    * ADSP-BF533/2/1 Sil. Rev. < 0.3     *
    * 05000070 - SDRAM PSSE bit cannot   *
    * be set again after SDRAM Powerup   *
    * Please check if required to enable *
    *************************************/
    //#define WORKAROUND_05000070
    #include "ezkitBF533_initcode.h"
#endif

#if defined __ADSPBF537__
    #include "ezkitBF537_initcode.h"
#endif

#if defined __ADSPBF538__
    #include "ezkitBF538_initcode.h"
#endif

#if defined __ADSPBF561__
    #include "ezkitBF561_initcode.h"
#endif

#if defined __ADSPBF548__
    #include "ezkitBF548_initcode.h"
#endif


/*****************************************************************************
 Symbolic constants / definitions
******************************************************************************/

#define SYSCFG_VAL              0x30                        /* Program this value into SYSCFG register */

/*****************************************************************************
 Functions
******************************************************************************/

.SECTION L1_startup;

/*****************************************************************************
 Prototypes
******************************************************************************/

.GLOBAL __reset;

/****************************************************************************
 Function:      __reset
 
 Description:   Initialize System Configuration Register
*****************************************************************************/

__reset:

/****************************************************************************
 Description:   Set up supervisor stack
****************************************************************************/

IMM32(SP,0xFFB00800);
FP = SP;


/****************************************************************************
 Description:   Initialize System Configuration Register
****************************************************************************/

R7 = SYSCFG_VAL;
SYSCFG = R7;
IMM32(P5,SYS_MMR_BASE);


/*****************************************************************************
 Pre-Init Section
******************************************************************************/

/************************************************
* System Interrupt Wakeup Registers (SIC_IWRx)  *
* Required for PLL_CTL and VR_CTL configuration *
* Processor idle and wake up after PLL_LOCKCNT  *
* expired.Reset = 0xFFFF FFFF                   *
* disable all other interrupt sources in the    *
* meantime to disable unwanted wake up events   *
************************************************/

#if defined (SET_PLL) || defined (EN_SDRAM)

R7 = IWR_DISABLE_ALL;
#if defined __ADSPBF54x__
[P5 + lo(SIC_IWR2)] = R7;
#endif
#if defined (__ADSPBF52x__) || defined (__ADSPBF538__) || defined (__ADSPBF54x__) || defined (__ADSPBF561__)
[P5 + lo(SIC_IWR1)] = R7;
#endif
R7 = IRQ_PLL_WAKEUP;
[P5 + lo(SIC_IWR0)] = R7;  /* enable PLL Wakeup Interrupt */

#endif /* defined (SET_PLL) || defined (EN_SDRAM) */


/*****************************************************************************************
 Description:   Dynamic Power Managment
*****************************************************************************************/
DPM:
#if defined SET_PLL
    call __full_on;
#endif /* SET_PLL */


/*****************************************************************************
 Description:   Configure EBIU
******************************************************************************/
EBIU:


/*****************************************************************************
 Description:   Configure and enable asynchronous memory
******************************************************************************/
#if defined EN_ASYNC_MEM

/**********************************************
* Asynchronous Memory Bank Control 0 Register *
**********************************************/

IMM32(R7,EBIU_AMBCTL0_VAL);
[P5 + lo(EBIU_AMBCTL0)] = R7;


/**********************************************
* Asynchronous Memory Bank Control 1 Register *
**********************************************/

IMM32(R7,EBIU_AMBCTL1_VAL);
[P5 + lo(EBIU_AMBCTL1)] = R7;


/**********************************************
* Asynchronous Memory Global Control Register *
***********************************************/

R7.L = EBIU_AMGCTL_VAL;
w[P5 + lo(EBIU_AMGCTL)] = R7;

#endif /* EN_ASYNC_MEM */


/*****************************************************************************
Description:   Configure and enable SDRAM
******************************************************************************/
#if defined EN_SDRSDRAM

#if defined WORKAROUND_05000070
    R7 = w[P5 + lo(EBIU_SDSTAT)];
    CC = bittst(R7, bitpos(SDRS));
    if !CC jump skip_sdram_enable;
#endif


/*************************************
* SDRAM Memory Bank Control Register *
**************************************/

#if defined __ADSPBF561__
    IMM32(R7,EBIU_SDBCTL_VAL);
    [P5 + lo(EBIU_SDBCTL)] = R7;
#else
    R7.L = EBIU_SDBCTL_VAL;
    w[P5 + lo(EBIU_SDBCTL)] = R7;
#endif


/***************************************
* SDRAM Memory Global Control Register *
***************************************/

IMM32(R7,EBIU_SDGCTL_VAL);
[P5 + lo(EBIU_SDGCTL)] = R7;


#if defined WORKAROUND_05000070 /* see 'ezkitBF533_initcode.h' for more details */
    skip_sdram_enable: nop;
#endif


/***************************************************************************
* SDRAM Refresh Rate Control Register                                      *
* RDIV has always to be re-calculated according to the actual system clock *
* You can either use the defined value that fits to the PCC settings or    *
* call __get_rdiv to calculate RDIV 'on the fly'                            *
***************************************************************************/

#if defined __PASEBF532__
    IMM32(R0,EBIU_SDRRC_VAL);
#else
    call __get_rdiv;
#endif
w[P5 + lo(EBIU_SDRRC)] = R0;

#endif /* EN_SDRSDRAM */


/*****************************************************************************
 Description:   Configure and enable DDR SDRAM
******************************************************************************/
#if defined EN_DDRSDRAM


/*************************
* Reset Control Register *
**************************/

R7 = w[P5 + lo(EBIU_RSTCTL)];
bitset(R7,bitpos(DDRSRESET));
w[P5 + lo(EBIU_RSTCTL)] = R7;


/****************************
* Memory Control 0 Register *
****************************/

IMM32(R7,EBIU_DDRCTL0_VAL);
[P5 + lo(EBIU_DDRCTL0)] = R7;


/****************************
* Memory Control 1 Register *
****************************/

IMM32(R7,EBIU_DDRCTL1_VAL);
[P5 + lo(EBIU_DDRCTL1)] = R7;


/****************************
* Memory Control 2 Register *
****************************/

IMM32(R7,EBIU_DDRCTL2_VAL);
[P5 + lo(EBIU_DDRCTL2)] = R7;


#endif /* EN_DDRSDRAM */


/*****************************************************************************
 End of Configure EBIU
******************************************************************************/


#if defined (SET_PLL) || defined (EN_SDRAM)

/*********************************************
* Restore System Interrupt Wakeup Registers  *
*********************************************/

R7 = IWR_ENABLE_ALL;

[P5 + lo(SIC_IWR0)] = R7;
#if defined (__ADSPBF52x__) || defined (__ADSPBF538__) || defined (__ADSPBF54x__) || defined (__ADSPBF561__)
[P5 + lo(SIC_IWR1)] = R7;
#endif
#if defined __ADSPBF54x__
[P5 + lo(SIC_IWR2)] = R7;
#endif

#endif /* defined (SET_PLL) || defined (EN_SDRAM) */


/****************************************************************************
 Description:   Configure Default Service Handlers
*****************************************************************************/

IMM32(P5,EVT2);
IMM32(R7,nmi_handler);
[P5++] = R7;
IMM32(R7,evx_handler);
[P5++] = R7;
/* IVT4 isn't used */
[P5++] = R7;
IMM32(R7,ivhw_handler);
[P5++] = R7;
IMM32(R7,ivtmr_handler);
[P5++] = R7;
IMM32(R7,ivg7_handler);
[P5++] = R7;
IMM32(R7,ivg8_handler);
[P5++] = R7;
IMM32(R7,ivg9_handler);
[P5++] = R7;
IMM32(R7,ivg10_handler);
[P5++] = R7;
IMM32(R7,ivg11_handler);
[P5++] = R7;
IMM32(R7,ivg12_handler);
[P5++] = R7;
IMM32(R7,ivg13_handler);
[P5++] = R7;
IMM32(R7,ivg14_handler);
[P5++] = R7;
IMM32(R7,ivg15_handler);
[P5++] = R7;

/****************************************************************************
 Description:   Configure and Enable CPLB / Cache
*****************************************************************************/

#if defined INSTR_CACHE_EN
    call __icplb_init;
#endif
#if defined DATA_CACHE_EN
    call __dcplb_init;
#endif


/****************************************************************************
 Description:   Leave startup code either to User or Supervisor Mode
                jump to main routine
*****************************************************************************/

IMM32(P5,_main);
RETI = P5;

#ifndef RUN_USER
    /* Enable Interrupt 15 and go to SUPERVISOR mode */
    IMM32(P5,EVT15);
    IMM32(R7,ivg15_handler.supervisor);
    [P5] = R7;
    R7 = 0x8000(z);
    sti R7;
    raise 15;
#endif

/* Return from RESET (IVG2) to either USER (no IVG) or SUPERVISOR (IVG15) */
rti;

#ifndef RUN_USER
    ivg15_handler.supervisor:
    [--SP] = RETI;
    IMM32(P5,_main);
    jump (P5);
#endif /* not RUN_USER */


/****************************************************************************
 Description:  Configure default Interrupt / Exception Handler
*****************************************************************************/

nmi_handler: EMUEXCPT;
nmi_handler.fail: jump nmi_handler.fail;


evx_handler:            /* Exception Handler 3 */
#if defined DATA_CACHE_EN

    /**********************
    * Verify a DCPLB miss *
    **********************/

    [--SP] = (R7:6);

    R7 = SEQSTAT;
    R6 = 0x3F;
    R7 = R7 & R6;
    R6 = 0x26; /* Data access CPLB miss */
    CC = R7 == R6;

    (R7:6) = [SP++];
    if !CC jump skip_DCPLB_MISS;


    [--SP] = (R7:7,P5:5);

    IMM32(P5,EVT14);
    IMM32(R7,__dcplb_handler);
    [P5] = R7;
    IMM32(P5,IMASK);
    R7 = [P5];
    bitset (R7,bitpos(EVT_IVG14));
    [P5] = R7;
    raise (bitpos(EVT_IVG14));

    (R7:7,P5:5) = [SP++];
    rtx;

    skip_DCPLB_MISS:

#endif

#if defined INSTR_CACHE_EN

    /**********************
    * Verify a ICPLB miss *
    **********************/

    [--SP] = (R7:6);

    R7 = SEQSTAT;
    R6 = 0x3F;
    R7 = R7 & R6;
    R6 = 0x2C; /* Instruction access CPLB miss */
    CC = R7 == R6;

    (R7:6) = [SP++];
    if !CC jump skip_ICPLB_MISS;


    [--SP] = (R7:7,P5:5);

    IMM32(P5,EVT14);
    IMM32(R7,__icplb_handler);
    [P5] = R7;
    IMM32(P5,IMASK);
    R7 = [P5];
    bitset (R7,bitpos(EVT_IVG14));
    [P5] = R7;
    raise (bitpos(EVT_IVG14));

    (R7:7,P5:5) = [SP++];
    rtx;

    skip_ICPLB_MISS:

#endif
    
/********************************************
* I don't know how to handle this exception *
********************************************/

EMUEXCPT;
evx_handler.fail: jump evx_handler.fail;

evx_handler.END:


ivhw_handler: rti;
EMUEXCPT;
ivhw_handler.fail: jump ivhw_handler.fail;

ivtmr_handler:
EMUEXCPT;
ivtmr_handler.fail: jump ivtmr_handler.fail;

ivg7_handler:
EMUEXCPT;
ivg7_handler.fail: jump ivg7_handler.fail;

ivg8_handler:
EMUEXCPT;
ivg8_handler.fail: jump ivg8_handler.fail;

ivg9_handler:
EMUEXCPT;
ivg9_handler.fail: jump ivg9_handler.fail;

ivg10_handler:
EMUEXCPT;
ivg10_handler.fail: jump ivg10_handler.fail;

ivg11_handler:
EMUEXCPT;
ivg11_handler.fail: jump ivg11_handler.fail;

ivg12_handler:
EMUEXCPT;
ivg12_handler.fail: jump ivg12_handler.fail;

ivg13_handler:
EMUEXCPT;
ivg13_handler.fail: jump ivg13_handler.fail;

ivg14_handler:
EMUEXCPT;
ivg14_handler.fail: jump ivg14_handler.fail;

ivg15_handler:
EMUEXCPT;
ivg15_handler.fail: jump ivg15_handler.fail;


__reset.end:


/*****************************************************************************
 EOF
******************************************************************************/
